import { TextLineStream } from "https://deno.land/std@0.208.0/streams/mod.ts";

export function channel<T>() {
	const inner = {
		queue: [] as T[],
		resolver: null as (() => void) | null,
	};
	const send = (item: T) => {
		inner.queue.push(item);
		if (inner.resolver === null) return;
		inner.resolver();
		inner.resolver = null;
	};
	const receive = async () => {
		if (inner.queue.length < 1) await new Promise<void>((res) => inner.resolver = res);
		const [head] = inner.queue.splice(0, 1);
		return head;
	};
	return { send, receive };
}

export function* range(from: number, to: number) {
	while (from < to) yield from++;
}

export async function exists(path: string) {
	try {
		await Deno.stat(path);
		return true;
	} catch (_) {
		return false;
	}
}

export async function next<T>(readable: ReadableStream<T>) {
	for await (const item of readable) return item;
}

export function lines(readable: ReadableStream<Uint8Array>) {
	return readable
		.pipeThrough(new TextDecoderStream())
		.pipeThrough(new TextLineStream());
}
